<?php
/**
 * @package J2Store
 * @copyright Copyright (c)2014-17 Ramesh Elamathi / J2Store.org
 * @license GNU GPL v3 or later
 */
// No direct access to this file
defined ( '_JEXEC' ) or die ();
class J2Cart {

	public static $instance = null;
	protected $state;

	public function __construct($properties=null) {

	}

	public static function getInstance(array $config = array())
	{
		if (!self::$instance)
		{
			self::$instance = new self($config);
		}

		return self::$instance;
	}

	public function getCartList() {

		$model = F0FModel::getTmpInstance('Carts', 'J2StoreModel');

		$session = JFactory::getSession();
		$user = JFactory::getUser();

		if( !$user->id ) // saves session id (will be needed after logging in)
		{
			$session->set( 'old_sessionid', $session->getId(), 'j2store' );
		}

		$state = array();

		$state['filter_user'] = $user->id;
		if (empty($user->id))
		{
			$state['filter_session'] = $session->getId();
		}

		foreach ($state as $key=>$value)
		{
			$model->setState( $key, $value );
		}

		$items  = $model->getList();

		return $items;
	}

	public function getSubtotal($items) {
		$subtotal = 0;
		if (! isset ( $items ) && count ( $items ) < 1)
			return $subtotal;

		foreach($items as $item) {
			$subtotal += $item->product_subtotal;
		}
		return $subtotal;
	}

	public function getCartTaxTotal($items) {
		$taxtotal = 0;
		if (! isset ( $items ) && count ( $items ) < 1)
			return $taxtotal;

		foreach($items as $item) {
			if(isset($item->taxes) && isset($item->taxes->taxtotal)) {
				$taxtotal += $item->taxes->taxtotal;
			}
		}
		return $taxtotal;
	}

	public static function getTaxes($items) {

		$tax_data = array();

		foreach ($items as $item) {
			if ($item->orderitem_taxprofile_id) {

				$tax_rates = $item->taxes->taxes;

				foreach ($tax_rates as $taxrate_id=>$tax_rate) {
					if (!isset($tax_data[$taxrate_id])) {
						$tax_data[$taxrate_id]['name'] = $tax_rate['name'];
						$tax_data[$taxrate_id]['rate'] = $tax_rate['rate'];
						$tax_data[$taxrate_id]['total'] = ($tax_rate['amount'] * $item->orderitem_quantity);
					} else {
						$tax_data[$taxrate_id]['name'] = $tax_rate['name'];
						$tax_data[$taxrate_id]['rate'] = $tax_rate['rate'];
						$tax_data[$taxrate_id]['total'] += ($tax_rate['amount'] * $item->orderitem_quantity);
					}
				}
			}
		}
		return $tax_data;
	}


	public function getCartTotalWeight($items) {

		$weight_total = 0;
		if (! isset ( $items ) && count ( $items ) < 1) return $weight_total;

		foreach($items as $item) {
			//only when shipping is enabled
			if(isset($item->shipping) && $item->shipping == 1) {
				$weight_total += $item->weight_total;
			}
		}
		return $weight_total;
	}

	public function removeCartItem($cart_id) {

	}

	public function getImage($type, $product_id) {

	}

	function resetCart( $session_id, $user_id )
	{

		$session = JFactory::getSession();
		$user = JFactory::getUser();

		//get cart items based on old session id
		$model = F0FModel::getTmpInstance('Carts', 'J2StoreModel');
		$model->setState( 'filter_user_leq', '0' );
		$model->setState( 'filter_session', $session_id );
		$cartitems  = $model->getList();

		//delete the items with old session id
		$this->deleteSessionCartItems( $session_id );

		if (!empty($cartitems))
		{
			F0FTable::addIncludePath( JPATH_ADMINISTRATOR.'/components/com_j2store/tables' );

			foreach ($cartitems as $cartitem)
			{
				$keynames = array();
				$keynames['user_id'] = $user_id;
				$keynames['product_id'] = $cartitem->product_id;
				$keynames['variant_id'] = $cartitem->variant_id;
				$keynames['product_options'] = $cartitem->product_options;

				$table = F0FTable::getInstance( 'Carts', 'J2StoreTable' )->getClone();

				$table->product_id = $cartitem->product_id;
				$table->variant_id = $cartitem->variant_id;
				if ($table->load($keynames))
				{
					// the quantity as set in the session takes precedence
					$table->product_qty = $cartitem->product_qty;
				}
				else
				{
					foreach($cartitem as $key=>$value)
					{
						if(property_exists($table, $key))
						{
							$table->set($key, $value);
						}
					}
					// this is a new cartitem, so set cart_id = 0
					$table->j2store_cart_id = '0';
				}

				$table->user_id = $user_id;
				$table->session_id = $session->getId();

				if (!$table->store())
				{
					JError::raiseNotice($table->getError());
				}
				$table->j2store_cart_id = '0';
			}
		}
	}

	function updateSession( $user_id, $session_id )
	{
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);

		$query->update( "#__j2store_carts" );
		$query->set( $db->qn('session_id')." = ".$db->q($session_id) );
		$query->where( $db->qn('user_id')." <= ".$db->q($user_id) );
		$db->setQuery( (string) $query );
		if (!$db->query())
		{
			$this->setError( $db->getErrorMsg() );
			return false;
		}
		return true;
	}


	function deleteSessionCartItems( $session_id )
	{
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);

		$query->delete();
		$query->from( "#__j2store_carts" );
		$query->where( $db->qn('session_id')." = ".$db->q($session_id) );
		$query->where( $db->qn('user_id')." <= 0 ");
		$db->setQuery( (string) $query );
		if (!$db->query())
		{
			$this->setError( $db->getErrorMsg() );
			return false;
		}
		return true;
	}

	public static function emptyCart( $order_id )
	{
		$app = JFactory::getApplication();
		$cart = F0FTable::getAnInstance( 'Cart', 'J2StoreTable' );
		$order = F0FTable::getAnInstance('Order', 'J2StoreTable');
		$order->load(array('order_id'=>$order_id));
		if (!empty($order->order_id))
		{
			$orderitems = $order->getItems();
			// foreach orderitem
			foreach ($orderitems as $orderitem)
			{
				// remove from user's cart
				$cart->delete($orderitem->cart_id);
				$app->triggerEvent( 'onJ2StoreRemoveOrderItem', array( $orderitem ) );
			}
		}
	}
}