<?php
/**
 * @package J2Store
 * @copyright Copyright (c)2014-17 Ramesh Elamathi / J2Store.org
 * @license GNU GPL v3 or later
 */
// No direct access to this file
defined('_JEXEC') or die;

require_once JPATH_ADMINISTRATOR.'/components/com_j2store/models/behavior/autoload.php';

class J2StoreModelCarts extends F0FModel {

	protected $default_behaviors = array('filters', 'cartdefault');
	protected  $_rawData = null;
	private $behavior_prefix = 'cart';
	protected $_cartitems = null;

	function __construct($config = array()) {
		  parent::__construct($config);
	}

	public function addCartItem() {

		$app = JFactory::getApplication();
		$errors = array();
		$json = new JObject();

		//first check if it has product id.
		$product_id = $this->input->get('product_id');

		if(!isset($product_id)) {
			$errors['error'] = JText::_('J2STORE_PRODUCT_NOT_FOUND');
			return $errors;
		}

		//product found. Load it
		$product = F0FModel::getTmpInstance('Products', 'J2StoreModel')->getItem($product_id);
		
		if(($product->visibility !=1) || ($product->enabled !=1) ) {
			$errors['error'] = array('error'=>JText::_('J2STORE_PRODUCT_NOT_ENABLED_CANNOT_ADDTOCART'));
			return $errors;
		}

		if($product->j2store_product_id != $product_id) {
			//so sorry. Data fetched does not match the product id
			$errors['error'] = JText::_('J2STORE_PRODUCT_NOT_FOUND');
			return $errors;
		}

		// import the j2store plugins for triggers in behaviour
		JPluginHelper::importPlugin ('j2store');
		
		//all ok. Fire model dispatcher

		if($product->product_type) {
			$this->addBehavior($this->behavior_prefix.$product->product_type);
		}else {
			$this->addBehavior($this->behavior_prefix.'simple');
		}

		try
		{
			// Call the behaviors
			$result = $this->modelDispatcher->trigger('onBeforeAddCartItem', array(&$this, $product, &$json ));
		}
		catch (Exception $e)
		{
			// Oops, an exception occured!
			$this->setError($e->getMessage());
		}

		return $json->result;
	}

	public function addItem($item) {

		$session = JFactory::getSession();
		$user = JFactory::getUser();

		$keynames = array();
		$item->user_id = (empty($item->user_id)) ? $user->id : $item->user_id;
		$keynames['user_id'] = $item->user_id;
		if (empty($item->user_id))
		{
			$keynames['session_id'] = $session->getId();
		}
		$keynames['variant_id'] = $item->variant_id;
		$keynames['product_options'] = $item->product_options;

		$table = F0FTable::getInstance('Cart', 'J2StoreTable');

		$table->product_id = $item->product_id;
		$table->variant_id = $item->variant_id;
		$table->product_type = $item->product_type;

		if ($table->load( $keynames) )
		{
			//if an item exists, we just add the quantity. Even if it does not
			$table->product_qty = $table->product_qty + $item->product_qty;
		}
		else
		{
			foreach($item as $key=>$value)
			{
				if(property_exists($table, $key))
				{
					$table->set($key, $value);
				}
			}
		}
		//get the IP of the customer
		$table->customer_ip = $_SERVER['REMOTE_ADDR'];
		$table->session_id = $session->getId();

		if ($table->store())
		{
			try
			{
				// Call the behaviors
				$result = $this->modelDispatcher->trigger('onAfterAddCartItem', array(&$this, &$table));
			}
			catch (Exception $e)
			{
				// Oops, an exception occured!
				$this->setError($e->getMessage());
				return false;
			}
			return $table;
		} else {
			return false;
		}

		return false;
	}

	protected function _buildQueryWhere(&$query)
	{
		$filter_user      = $this->getState('filter_user');
		$filter_user_leq  = $this->getState('filter_user_leq');
		$filter_session   = $this->getState('filter_session');
		$filter_product   = $this->getState('filter_product');
		$filter_variant   = $this->getState('filter_variant');
		$filter_date_from	= $this->getState('filter_date_from');
		$filter_date_to		= $this->getState('filter_date_to');
		$filter_name	= $this->getState('filter_name');

		if (strlen($filter_user))
		{
			$query->where('tbl.user_id = '.$this->_db->Quote($filter_user));
		}

		if (strlen($filter_user_leq))
		{
			$query->where('tbl.user_id <= '.$this->_db->Quote($filter_user_leq));
		}


		if (strlen($filter_session))
		{
			$query->where( "tbl.session_id = ".$this->_db->Quote($filter_session));
		}

		if (!empty($filter_product))
		{
			$query->where('tbl.product_id = '.(int) $filter_product);
		}

		if (!empty($filter_variant))
		{
			$query->where('tbl.variant_id = '.(int) $filter_variant);
		}

		if (strlen($filter_date_from))
		{
			$query->where("tbl.modified_on >= '".$filter_date_from."'");
		}

		if (strlen($filter_date_to))
		{
			$query->where("tbl.modified_on <= '".$filter_date_to."'");
		}

		if (strlen($filter_name))
		{
			$key	= $this->_db->Quote('%'.$this->_db->getEscaped( trim( strtolower( $filter_name ) ) ).'%');
			$query->where('LOWER(variant.product_name) LIKE '.$key);
		}
		//Do we really need the following checks?
		$query->where('product.enabled =1');
		$query->where('product.visibility=1');

	}

	protected function _buildQueryFields(&$query)
	{
		$field = array();

		$field[] = " variant.j2store_variant_id";
		$field[] = " variant.sku";
		$field[] = " variant.price";
		$field[] = " variant.shipping";
		$field[] = " variant.weight";
		$field[] = " variant.length";
		$field[] = " variant.width";
		$field[] = " variant.height";
		$field[] = " variant.length_class_id";
		$field[] = " variant.weight_class_id";
		$field[] = " variant.quantity_restriction ";
		$field[] = " variant.min_sale_qty";
		$field[] = " variant.max_sale_qty";
		$field[] = " variant.use_store_config_min_sale_qty";
		$field[] = " variant.use_store_config_max_sale_qty";
		$field[] = " variant.use_store_config_notify_qty";
		$field[] = " variant.manage_stock";
		$field[] = " variant.allow_backorder";

		$field[] = " product.product_type";
		$field[] = " product.product_source";
		$field[] = " product.product_source_id";
		$field[] = " product.has_options";
		$field[] = " product.up_sells";
		$field[] = " product.cross_sells";
		$field[] = " product.taxprofile_id";
		$field[] = " product.vendor_id";
		$field[] = " product.manufacturer_id";
		$field[] = " product.params as product_params";

		$field[] = " product.visibility";

		$field[] = " productimage.thumb_image";
		$field[] = " stock.quantity as available_quantity";

		$query->select('tbl.*');
		$query->select( $field );
	}

	protected function _buildQueryJoins(&$query)
	{
		$query->join('INNER', '#__j2store_variants AS variant ON tbl.variant_id = variant.j2store_variant_id');
		$query->join('INNER', '#__j2store_products AS product ON tbl.product_id = product.j2store_product_id');
		$query->join('LEFT OUTER', '#__j2store_productimages AS productimage ON product.j2store_product_id = productimage.product_id');
		$query->join('LEFT OUTER', '#__j2store_productquantities AS stock ON variant.j2store_variant_id = stock.variant_id');
	}

	 public function buildQuery($overrideLimits=false) {
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)->from('#__j2store_carts as tbl');
		$this->_buildQueryFields($query);
		$this->_buildQueryJoins($query);
		$this->_buildQueryWhere($query);
		$query->group('tbl.j2store_cart_id');
		return $query;
	}


	public function getItems() {
		
		//validate the items before you do this
		$this->validate_items();
		$session = JFactory::getSession ();
		$user = JFactory::getUser ();		
		if (empty( $this->_cartitems)) {		
			if (! $user->id) 			// saves session id (will be needed after logging in)
			{
				$session->set ( 'old_sessionid', $session->getId (), 'j2store' );
			}

			$state = array ();

			$state ['filter_user'] = $user->id;
			if (empty ( $user->id )) {
				$state ['filter_session'] = $session->getId ();
			}

			foreach ( $state as $key => $value ) {
				$this->setState ( $key, $value );
			}

			$items = $this->getList ();

			// now process the items
			$this->getCartItems ( $items );
			
			$this->_cartitems = $items;
		}
		return $this->_cartitems;
	}

	public function getCartItems(&$items) {

			$params = J2Store::config();
			foreach($items as $item) {

				//all ok. Fire model dispatcher

				if($item->product_type) {
					$this->addBehavior($this->behavior_prefix.$item->product_type);
				}else {
					$this->addBehavior($this->behavior_prefix.'simple');
				}

				//run model behaviors
				try
				{
					// Call the behaviors
					$this->modelDispatcher->trigger('onGetCartItems', array(&$this, &$item));
				}
				catch (Exception $e)
				{
					// Oops, an exception occured!
					$this->setError($e->getMessage());
					return false;
				}


			} // cart item loops
			
			J2Store::plugin()->event('AfterGetCartItems', array(&$items));

	} //end of cart item function

	function update() {
		$app = JFactory::getApplication ();
		$post = $app->input->getArray ( $_POST );
		$productHelper = J2Store::product ();

		$json = array ();

		foreach ( $post ['quantities'] as $cart_id => $quantity ) {

			$cartitem = F0FModel::getTmpInstance ( 'Carts', 'J2StoreModel' )->getItem ( $cart_id );

			// get the difference quantity

			if ($this->validate ($cartitem, $quantity ) === false) {
				// an error occured. Return it
				$json ['error'] = $this->getError();
				continue; // exit from the loop
			}

			// validation successful. Update cart
			$cart = F0FTable::getInstance ( 'Cart', 'J2StoreTable' );
			$cart->load ( $cart_id );
			if (empty ( $quantity ) || $quantity < 1) {
				// the user wants to remove the item from cart. so remove it

				$item = new JObject ();
				$item->product_id = $cartitem->product_id;
				$item->variant_id = $cartitem->variant_id;
				$item->product_type = $cartitem->product_type;
				$item->product_options = $cartitem->product_options;

				$cart->delete ( $cart_id );
				$app->triggerEvent ( 'onJ2StoreRemoveFromCart', array (
						$item
				) );
			} else {
				$cart->product_qty = J2Store::utilities()->stock_qty($quantity);
				$cart->store ();
			}
		}
		return $json;
	}

	function deleteItem() {

		// TODO we should be removing promotions as well
		$app = JFactory::getApplication();
		$cart_id = $this->input->get ( 'cart_id' );
		$cart = F0FTable::getInstance ( 'Cart', 'J2StoreTable' );

		// the user wants to remove the item from cart. so remove it
		if ($cart->load ( $cart_id )) {
			$item = new JObject ();
			$item->product_id = $cart->product_id;
			$item->variant_id = $cart->variant_id;
			$item->product_options = $cart->product_options;

			if ($cart->delete ( $cart_id )) {
				$app->triggerEvent ( 'onJ2StoreRemoveFromCart', array (
						$item
				) );
				return true;
			} else {
				$this->setError ( JText::_ ( 'J2STORE_CART_DELETE_ERROR' ) );
				return false;
			}
		} else {
			$this->setError ( JText::_ ( 'J2STORE_CART_DELETE_ERROR' ) );
			return false;
		}
	}

	function validate($cartitem, $quantity) {

		$json = new JObject();

		if($cartitem->product_type) {
			$this->addBehavior($this->behavior_prefix.$cartitem->product_type);
		}else {
			$this->addBehavior($this->behavior_prefix.'simple');
		}

		try
		{
			// Call the behaviors
			$result = $this->modelDispatcher->trigger('onValidateCart', array(&$this, $cartitem, $quantity));
		}
		catch (Exception $e)
		{
			// Oops, an exception occured!
			$result = false;
			$this->setError($e->getMessage());
		}
		return $result;
	}
	
	function validate_items() {
		$user = JFactory::getUser();
		$session = JFactory::getSession();
		if (is_null ( $this->_cartitems)) {
			if (! $user->id) 			// saves session id (will be needed after logging in)
			{
				$session->set ( 'old_sessionid', $session->getId (), 'j2store' );
			}
		
			$state = array ();
		
			$state ['filter_user'] = $user->id;
			if (empty ( $user->id )) {
				$state ['filter_session'] = $session->getId ();
			}
		
			foreach ( $state as $key => $value ) {
				$this->setState ( $key, $value );
			}
		
			$cartitems = $this->getList ();
		}else {
			$cartitems = $this->_cartitems;
		}
		
		$product_helper = J2Store::product();
		foreach($cartitems as $cartitem) {
			if(!$product_helper->setId($cartitem->product_id)->exists()) {
				$this->setIds(array($cartitem->j2store_cart_id))->delete();
				JFactory::getApplication()->enqueueMessage(JText::_('J2STORE_CART_ITEM_UNAVAILABLE_REMOVED'), 'warning');
			}
		}
		
	}
	
	function getCartUrl() {

		$url = JRoute::_('index.php?option=com_j2store&view=carts');
		//allow plugins to alter the cart link.
		//This allows 3rd-party developers to create different checkout steps for j2store
		//Example. The plugin can return a completely different link: index.php?option=com_example&view=myview
		J2Store::plugin()->event('GetCartLink', array(&$url));
		return $url;
	}
	
	function getCheckoutUrl() {
	
		$url = JRoute::_('index.php?option=com_j2store&view=checkout');
		//allow plugins to alter the checkout link.
		//This allows 3rd-party developers to create different checkout steps for j2store
		//Example. The plugin can return a completely different link: index.php?option=com_example&view=myview
	
		J2Store::plugin()->event('GetCheckoutLink', array(&$url));
		return $url;
	}
	
	

	function getContinueShoppingUrl() {

		$params = J2Store::config();
		$type = $params->get('config_continue_shopping_page', 'previous');

		$item = new JObject();
		$item->type = $type;

		switch($type) {
			case 'previous':
			default:
				$item->url = '';
				break;

			case 'menu':
				$url = '';
				//get the menu item id
				$menu_itemid = $params->get('continue_shopping_page_menu', '');
				if(empty($menu_itemid)) {
					$item->url = '';
					$item->type = 'previous';
				} else {
					
					$application = JFactory::getApplication();
					$menu = $application->getMenu('site');
					$menu_item = $menu->getItem($menu_itemid);
					
					if(is_object($menu_item)) {
						
						// we have the menu item. See if language associations are there
						JLoader::register('MenusHelper', JPATH_ADMINISTRATOR . '/components/com_menus/helpers/menus.php');						
						try {
							$associations = MenusHelper::getAssociations($menu_item->id);
						}catch (Exception $e) {
							$associations = array();
						}
												
						//get the current language code
						$tag = JFactory::getLanguage()->getTag();
						if(isset($associations[$tag])) {
							$cmenu = $menu->getItem($associations[$tag]);
						}else {
							$cmenu = $menu_item;
						}
						
						$link = JRoute::_($cmenu->link.'&Itemid='.$cmenu->id, false);
						if (JURI::isInternal($link)) {
							$url = $link; 
						}
					}
					if(empty($url)) {
						$item->url = '';
						$item->type = 'previous';
					} else {
						$item->url =$url;
					}	
				}

				break;

				case 'url':

					$custom_url = $params->get('config_continue_shopping_page_url', '');
					if(empty($custom_url)) {
						$item->url = '';
						$item->type = 'previous';
					} else {
						$item->url = $custom_url;
					}
					break;

		}

		//allow plugins to alter the checkout link.
		//This allows 3rd-party developers to create different checkout steps for j2store
		//Example. The plugin can return a completely different link

		J2Store::plugin()->event('GetContinueShoppingUrl', array(&$item));
		return $item;
	}


	public function validate_files($files = array()) {
		$app = JFactory::getApplication();
		$json = array();
		if(count($files) < 1) $files = $app->input->files->get('file');

		$upload_result = $this->uploadFile($files);
		if($upload_result == false) {
			$json['error'] = $this->getError();
		} else {

			$upload = F0FTable::getInstance('Upload', 'J2StoreTable');
			$upload->reset();
			$upload->j2store_upload_id = null;


			$jdate = new JDate();

			$upload_result['created_by'] = JFactory::getUser()->id;
			$upload_result['created_on'] = $jdate->toSql();
			$upload_result['enabled']    = 1;

			if(!$upload->save($upload_result)) {
				$json['error'] = JText::sprintf('J2STORE_UPLOAD_ERR_GENERIC_ERROR');
			}

		}

		if (!$json) {
			$json['name'] = $upload_result['original_name'];
			$json['code'] = $upload_result['mangled_name'];
			$json['success'] = JText::_('J2STORE_UPLOAD_SUCCESSFUL');
		}
		return $json;
	}

	protected function uploadFile($file, $checkUpload = true)
	{
		if (isset($file['name']))
        {
            JLoader::import('joomla.filesystem.file');

			// Can we upload this file type?
			if ($checkUpload)
			{

				if(!class_exists('MediaHelper')) {
					require_once(JPATH_ADMINISTRATOR.'/components/com_media/helpers/media.php');
				}

                $err   = '';
				$paths = array(JPATH_ROOT, JPATH_ADMINISTRATOR);
				$jlang = JFactory::getLanguage();
				$jlang->load('com_media', $paths[0], 'en-GB', true);
				$jlang->load('com_media', $paths[0], null, true);
				$jlang->load('com_media', $paths[1], 'en-GB', true);
				$jlang->load('com_media', $paths[1], null, true);

				if (!MediaHelper::canUpload($file, $err))
                {
					if (!empty($err))
					{
						$err = JText::_($err);
					}
					else
					{
						$app = JFactory::getApplication();
						$errors = $app->getMessageQueue();

						if (count($errors))
						{
							$error = array_pop($errors);
							$err = $error['message'];
						}
						else
						{
							$err = '';
						}
					}

					$content = file_get_contents($file['tmp_name']);
					if (preg_match('/\<\?php/i', $content)) {
						$err = JText::_('J2STORE_UPLOAD_FILE_PHP_TAGS');
					}

					if (!empty($err))
					{
						$this->setError(JText::_('J2STORE_UPLOAD_ERR_MEDIAHELPER_ERROR').' '.$err);
					}
					else
					{
						$this->setError(JText::_('J2STORE_UPLOAD_ERR_GENERIC_ERROR'));
					}

					return false;
				}
			}

			// Get a (very!) randomised name
            $serverkey = JFactory::getConfig()->get('secret','');
			$sig       = $file['name'].microtime().$serverkey;

			if(function_exists('sha256'))
            {
				$mangledname = sha256($sig);
			}
            elseif(function_exists('sha1'))
            {
				$mangledname = sha1($sig);
			}
            else
            {
				$mangledname = md5($sig);
			}
			$upload_folder_path = JPATH_ROOT.'/media/j2store/uploads';
			if(!JFolder::exists($upload_folder_path)) {
				if(!JFolder::create($upload_folder_path)) {
					$this->setError(JText::_('J2STORE_UPLOAD_ERROR_FOLDER_PERMISSION_ERROR'));
				}
			}
			//sanitize file name
			$filename = basename(preg_replace('/[^a-zA-Z0-9\.\-\s+]/', '', html_entity_decode($file['name'], ENT_QUOTES, 'UTF-8')));

			$name = $filename . '.' . md5(mt_rand());
			// ...and its full path
			$filepath = JPath::clean(JPATH_ROOT.'/media/j2store/uploads/'.$name);

			// If we have a name clash, abort the upload
			if (JFile::exists($filepath))
            {
				$this->setError(JText::_('J2STORE_UPLOAD_ERR_NAMECLASH'));
				return false;
			}

			// Do the upload
			if ($checkUpload)
			{
				if (!JFile::upload($file['tmp_name'], $filepath))
                {
					$this->setError(JText::_('J2STORE_UPLOAD_ERR_CANTJFILEUPLOAD'));
					return false;
				}
			}
			else
			{
				if (!JFile::copy($file['tmp_name'], $filepath))
				{
					$this->setError(JText::_('J2STORE_UPLOAD_ERR_CANTJFILEUPLOAD'));
					return false;
				}
			}

			// Get the MIME type
			if(function_exists('mime_content_type'))
            {
				$mime = mime_content_type($filepath);
			}
            elseif(function_exists('finfo_open'))
            {
				$finfo = finfo_open(FILEINFO_MIME_TYPE);
				$mime = finfo_file($finfo, $filepath);
			}
            else
            {
				$mime = 'application/octet-stream';
			}

			// Return the file info
			return array(
				'original_name'	=> $file['name'],
				'mangled_name'	=> $mangledname,
				'saved_name'	=> $name,
				'mime_type'			=> $mime
			);
		}
        else
        {
			$this->setError(JText::_('J2STORE_ATTACHMENTS_ERR_NOFILE'));

			return false;
		}
	}
}
